#include <algorithm>
#include <iostream>
#include <numeric>
#include <vector>
using namespace std;

typedef double (*Fun)(double);

class Composer {
private:
    const vector<Fun>& funs;
public:
    Composer(const vector<Fun>& fs) : funs(fs) {}
    double operator()(double x) const {
        double result = x;
        for (vector<Fun>::const_reverse_iterator p = funs.rbegin(); p != funs.rend(); ++p)
            result = (*p)(result);
        return result;
    }
};

double f(double x) {
    return x*x;
}

double g(double x) {
    return x+1;
}

double h(double x) {
    return x/3.0;
}

int main() {
    vector<Fun> funs;
    funs.push_back(h);
    funs.push_back(f);
    funs.push_back(g);
    Composer comp(funs);
    cout << comp(2.0);  // 3
}
