// Generalize the sequence type
#include <algorithm>
#include <iostream>
#include <iterator>
#include <list>
#include <numeric>
using namespace std;

template<class T, class Iter>
class Composer {
private:
    typedef reverse_iterator<Iter> RevIter;
    RevIter beg, end;
public:
    Composer(Iter b, Iter e) : beg(RevIter(e)), end(RevIter(b)) {}
    T operator()(T x) const {
        T result = x;
        for (RevIter p = beg; p != end; ++p)
            result = (*p)(result);
        return result;
    }
};

double f(double x) {
    return x*x;
}

double g(double x) {
    return x+1;
}

double h(double x) {
    return x/3.0;
}

int main() {
    typedef double (*Fun)(double);
    
    // Try a list
    list<Fun> funs;
    funs.push_back(h);
    funs.push_back(f);
    funs.push_back(g);
    Composer<double, list<Fun>::iterator > comp(funs.begin(), funs.end());
    cout << comp(2.0) << endl;

    // Try an array
    Fun funs2[] = {h,f,g};
    Composer<double, Fun*> comp2(funs2, funs2+3);
    cout << comp2(2.0) << endl;
}
