// Add a helper function
#include <algorithm>
#include <functional>
#include <iostream>
#include <iterator>
#include <list>
#include <numeric>
using namespace std;
using namespace tr1;

template<class Iter>
class Composer {
private:
    // Deduce function and return/argument types
    typedef typename iterator_traits<Iter>::value_type Fun;
    typedef typename Fun::result_type T;

    // Declare reverse iterators (see constructor for use)
    typedef reverse_iterator<Iter> RevIter;
    RevIter beg, end;

    // The function called by accumulate
    static T apply(T sofar, Fun f) {
        return f(sofar);
    }
public:
    Composer(Iter b, Iter e) : beg(RevIter(e)), end(RevIter(b)) {}
    T operator()(T x) const {
        return accumulate(beg, end, x, apply);  // Applicator
    }
};

// Regular functions
double f(double x) {
    return x*x;
}
double g(double x) {
    return x+1;
}
double h(double x) {
    return x/3.0;
}

template<class Iter>
Composer<Iter> compose(Iter b, Iter e) {
    return Composer<Iter>(b,e);
}

int main() {
    typedef std::tr1::function<double(double)> Fun;

    // Use a mixture of function objects and pointers
    std::list<Fun> funs;
    funs.push_back(bind2nd(divides<double>(), 3.0));
    funs.push_back(&f);    // Plain function
    funs.push_back(bind2nd(plus<double>(), 1.0));
    cout << compose(funs.begin(), funs.end())(2.0) << endl;
}

