// ieee.cpp: Test program for ieee.h
#include <cassert>
#include <iostream>
#include <limits>
#include "ieee.h"
using namespace std;

int main() {
   float x = 6.5f;
   double y = 6.5;
   cout << "Little endian? " << boolalpha << is_little() << noboolalpha << endl;

   assert(sign_bit(x) == 0);
   assert(sign_bit(y) == 0);
   assert(sign_bit(0.0f) == 0);
   assert(sign_bit(0.0) == 0);
   assert(sign_bit(-x) == 1);
   assert(sign_bit(-y) == 1);
   assert(sign(x) == 1);
   assert(sign(0.0f) == 0);
   assert(sign(-x) == -1);
   assert(sign(y) == 1);
   assert(sign(0.0) == 0);
   assert(sign(-y) == -1);
   assert(sign(-0.0f) == 0);
   assert(sign(-0.0) == 0);
   assert(exponent(x) == 2);
   assert(exponent(y) == 2);
   assert(exponent(16.5f) == 4);
   assert(exponent(16.5) == 4);
   assert(!is_normal(0.0f));
   assert(is_subnormal(0.0f));
   assert(!is_normal(0.0));
   assert(is_subnormal(0.0));
   assert(is_normal(6.5f));
   assert(!is_subnormal(6.5f));
   assert(is_normal(6.5));
   assert(!is_subnormal(6.5));
   assert(fraction(x) == 0.625f);
   assert(fraction(y) == 0.625);
   assert(fraction(0.0f) == 0.0f);
   assert(fraction(0.0) == 0.0);
   assert(mantissa(x) == 1.625f);
   assert(mantissa(y) == 1.625);
   assert(mantissa(0.0f) == 0.0f);
   assert(mantissa(0.0) == 0.0);
   float finf = numeric_limits<float>::infinity();
   assert(exponent(finf) == 128);
   float fnan = sqrt(-1.0);
   assert(exponent(fnan) == 128);
   assert(exponent(0.0f) == -126);
   assert(exponent(0.0) == -1022);
   assert(is_pos_infinity(finf));
   assert(!is_neg_infinity(finf));
   assert(!is_pos_infinity(-finf));
   assert(is_neg_infinity(-finf));
   assert(!is_finite(-finf));
   assert(is_finite(1.0));
   assert(is_finite(1.0f));
   assert(is_nan(fnan));
   assert(is_nan(-fnan));
   cout << "ulp(6.5f) == " << ulp(6.5f) << endl;
   cout << "ulp(6.5) == " << ulp(6.5) << endl;
   cout << "ulp(1.0f) == " << ulp(1.0f) << endl;
   cout << "ulp(1.0) == " << ulp(1.0) << endl;
   cout << "ulp(0.0f) == " << ulp(0.0f) << endl;
   cout << "ulp(0.0) == " << ulp(0.0) << endl;
   cout << "ulp(0.5f) == " << ulp(0.5f) << endl;
   cout << "ulp(0.5) == " << ulp(0.5) << endl;
   cout << "ulp(fmin) == " << ulp(numeric_limits<float>::denorm_min()) << endl;
   cout << "ulp(dmin) == " << ulp(numeric_limits<double>::denorm_min()) << endl;
   assert(copy_sign(6.5f, -1.0f) == -6.5f);
   assert(copy_sign(6.5, -1.0) == -6.5);
   assert(copy_sign(-6.5f, 1.0f) == 6.5f);
   assert(copy_sign(-6.5, 1.0) == 6.5);
   assert(copy_sign(finf, -1.0f) == -finf);
   assert(copy_sign(-finf, 1.0f) == finf);
}

/* Output:
Little endian? true
ulp(6.5f) == 4.76837e-07
ulp(6.5) == 8.88178e-16
ulp(1.0f) == 1.19209e-07
ulp(1.0) == 2.22045e-16
ulp(0.0f) == 1.4013e-45
ulp(0.0) == 4.94066e-324
ulp(0.5f) == 5.96046e-08
ulp(0.5) == 1.11022e-16
ulp(fmin) == 1.4013e-45
ulp(dmin) == 4.94066e-324
*/



