// fsin.cpp: Calculates sin(x) as a double. x is
// reduced to the interval [-pi/2, pi/2]. Uses
// the first 11 non-zero terms of the Taylor's Series
// (more would add nothing).
#include <iostream>
#include "../ieee.h"
using namespace std;

double pi = 3.14159265358979323846;
double pi1 = 3.1416015625;
double pi2 = -8.908910206761537356617e-6;
const int k = 11;

double mysine(double x) {
   int n = int(x/pi + sign(x));
   x = x - n*pi;
//   x = x - n*pi1;
//   x = x - n*pi2;
   double num = x;
   double den = 1.0;
   double sum = x;
   int fact = 1;  // The denominator contains fact!

   for (int i = 0; i < k; ++i) {
      num = -num*x*x;
      den *= ++fact;
      den *= ++fact;
      sum += num/den;
   }
   return (n%2) ? -sum : sum;
}

int main() {
    cout.precision(numeric_limits<double>::digits10);
    cout << mysine(0.0) << " (" << sin(0.0) << ")\n";
    cout << mysine(pi/2.0) << " (" << sin(pi/2.0) << ")\n";
    cout << mysine(pi) << " (" << sin(pi) << ")\n";
    cout << mysine(3.0*pi/2.0) << " (" << sin(3.0*pi/2.0) << ")\n";
    cout << mysine(22.0) << " (" << sin(22.0) << ")\n";
    cout << mysine(10.0e8) << " (" << sin(10.0e8) << ")\n";
}
