// epsilon.cpp: Extracts machine epsilon and other parms from the system headers
// You will note that the min and max values are different that what's expected!
// That will be explained later.
#include <cmath>
#include <iostream>
#include <limits>
#include <typeinfo>
using namespace std;

template<typename FType>
void printParms() {
   cout << "Parms for " << typeid(FType).name() << ":\n";
   int radix = numeric_limits<FType>::radix;
   int digits = numeric_limits<FType>::digits;
   cout << "base = " << radix << endl;
   cout << "precision = " << digits << endl;
   cout << "m = " << numeric_limits<FType>::min_exponent << endl;
   cout << "M = " << numeric_limits<FType>::max_exponent << endl;
   cout << "eps = " << numeric_limits<FType>::epsilon() << endl;
   cout << "wobble: [" << radix/2.0*pow(FType(radix), -digits) << ", ";
   cout << pow(FType(radix), 1-digits) << "]\n";
   cout << "min = " << numeric_limits<FType>::min() << endl;
   cout << "denorm_min = " << numeric_limits<FType>::denorm_min() << endl;
   cout << "max = " << numeric_limits<FType>::max() << endl;
   cout << endl;
}

int main() {
   printParms<float>();
   printParms<double>();
}

/* Output on VC++ .NET:
Parms for float:
base = 2
precision = 24
m = -125
M = 128
eps = 1.19209e-007
wobble: [5.96046e-008, 1.19209e-007]
min = 1.17549e-038
denorm_min = 1.4013e-045
max = 3.40282e+038

Parms for double:
base = 2
precision = 53
m = -1021
M = 1024
eps = 2.22045e-016
wobble: [1.11022e-016, 2.22045e-016]
min = 2.22507e-308
denorm_min = 4.94066e-324
max = 1.79769e+308
*/
