// fa3.cpp: Builds a FA transition table at runtime
#include <algorithm>
#include <cstddef>
#include <cstdlib>
#include <cstring>
#include <fstream>
#include <iostream>
#include <map>
#include <set>
#include <sstream>
#include <stdexcept>
#include <string>
#include <vector>
using namespace std;

bool recognize(const string& file, const string& str);

int main(int argc, char* argv[])
{
  if (argc < 2)
  {
    cout << "Incomplete argument list\n";
    return EXIT_FAILURE;
  }
  try
  {
    cout << boolalpha << recognize(argv[1], argv[2]);
  }
  catch (exception& e)
  {
    cout << e.what() << endl;
  }
}

bool recognize(const string& file, const string& str)
{
  // Open file
  ifstream inf(file.c_str());
  if (!inf)
    throw runtime_error("bad file name");

  // Read alphabet
  vector<char> alphabet;
  string line;
  getline(inf, line);
  istringstream is(line);
  char c;
  while (is >> c)
    alphabet.push_back(c);
  size_t letterCount = alphabet.size();

  // Read each row of transition table into map
  map<string, vector<string> > table;
  bool foundStart = false;
  string startState;
  while (getline(inf, line) && line[0] != '@')
  {
    istringstream is(line);

    // Read current state
    string currentState;
    is >> currentState;
    if (!is)
      throw runtime_error("Missing current state");
    if (!foundStart)
    {
      startState = currentState;
      foundStart = true;
    }

    // Read transitions for this state
    for (size_t i = 0; i < letterCount; ++i)
    {
      string nextState;
      is >> nextState;
      if (!is)
        throw runtime_error("missing transition for " + currentState);
      table[currentState].push_back(nextState);
    }
  }

  // Read success states
  set<string> accept;
  getline(inf, line);
  istringstream lastLine(line);
  string state;
  if (!inf)
    throw runtime_error("Missing accept state(s)");
  while (lastLine >> state)
    accept.insert(state);
  if (accept.size() == 0)
    cout << "No success states\n";

  // Test
  state = startState;
  for (size_t i = 0; i < str.size(); ++i)
  {
    char c = str[i];
    vector<string>& transitions = table[state];
    vector<char>::iterator letterPos = find(alphabet.begin(), alphabet.end(), c);
    if (letterPos == alphabet.end())
      throw runtime_error(string("Invalid input character ") + c);
    state = transitions[letterPos - alphabet.begin()];
  }
  return accept.find(state) != accept.end();
}

