// fa3.cpp: Builds a FA transition table at runtime
#include <algorithm>
#include <cassert>
#include <exception>
#include <fstream>
#include <iostream>
#include <iterator>
#include <map>
#include <set>
#include <sstream>
#include <string>
#include <vector>
using namespace std;

bool recognize(const string& file, const string& str) {
  // Open file
  ifstream inf(file.c_str());
  inf.exceptions(ios::failbit | ios::badbit);

  // Read alphabet (assume it's a single string - no spaces)
  string alphabet;
  getline(inf, alphabet);

  // Read each row of transition table into map
  map<string, vector<string> > table;
  string line;
  set<string> acceptStates;
  while (getline(inf, line) && line.size() > 0) {
    istringstream is(line);

    // Read current state
    string currentState;
    is >> currentState;
    if (currentState[currentState.size()-1] == '*') {
       currentState.resize(currentState.size()-1);
       acceptStates.insert(currentState);
    }
    
    // Read its transition states
    vector<string>& stateVector = table[currentState];
    copy(istream_iterator<string>(is), istream_iterator<string>(), back_inserter(stateVector));
  }
    
  // Test
  string state = "start";
  for (size_t i = 0; i < str.size(); ++i) {
    char c = str[i];
    vector<string>& transitions = table[state];
    size_t letterPos = alphabet.find(c);
    assert(letterPos != string::npos);
    state = transitions[letterPos];
  }
  return acceptStates.find(state) != acceptStates.end();
}

int main(int argc, char* argv[]) {
  if (argc >= 2)
     try {
       cout << boolalpha << recognize(argv[1], argv[2]);
     }
     catch (exception& e) {
       cout << e.what() << endl;
     }
}
