// findk.cpp: Finds the stopping term in the Taylor's series
// sufficient to maximize the precision of computing sin x.

#include <cassert>
#include <cmath>
#include <iostream>
#include <limits>
using namespace std;

// Template traits for pi
template<typename T>
struct pi_traits{
   static T pi() {return T();}
};

template<>
struct pi_traits<float> {
   static float pi() {return 3.14159265358979323846f;}
};

template<>
struct pi_traits<double> {
   static double pi() {return 3.14159265358979323846;}
};

// A factorial in floating-point (still requires integers!)
template<typename FType>
FType fact(FType n) {
   assert(n > 0 && n == ceil(n));
   return (n == 0 || n == 1) ? 1 : n*fact(n-1);
}

// Finds the first k for which (pi/2)^2k <= (eps*(2k+1)!)/2
template<typename FType>
void findk() {
   cout << typeid(FType).name() << endl;
   FType half_pi = pi_traits<FType>::pi() / 2;
   FType eps = numeric_limits<FType>::epsilon();
   for (int k = 0; k < 20; ++k) {
      FType lhs = pow(half_pi, 2*k);
      FType rhs = eps*fact(FType(2*k+1))/2;
      cout << k << ": " << lhs << ", " << rhs << endl;
      if (lhs <= rhs)
         break;
   }
   cout << endl;
}

int main() {
   findk<float>();
   findk<double>();
}

/* Output:
float
0: 1, 5.96046e-08
1: 2.4674, 3.57628e-07
2: 6.08807, 7.15256e-06
3: 15.0217, 0.000300407
4: 37.0646, 0.0216293
5: 91.4532, 2.37923
6: 225.652, 371.159

double
0: 1, 1.11022e-16
1: 2.4674, 6.66134e-16
2: 6.08807, 1.33227e-14
3: 15.0217, 5.59552e-13
4: 37.0646, 4.02878e-11
5: 91.4532, 4.43166e-09
6: 225.652, 6.91338e-07
7: 556.773, 0.000145181
8: 1373.78, 0.0394892
9: 3389.67, 13.5053
10: 8363.68, 5672.23
11: 20636.6, 2.87015e+06
*/
