// findk.cpp: Finds the stopping term in the Taylor's series
// sufficient to maximize the precision of computing sin x.
// It computes both sides of (pi/2)^(n+1) ~ eps/2*(n+1)!
// and stops when the left is less than the right.

#include <cassert>
#include <cmath>
#include <iostream>
#include <limits>
using namespace std;

template<typename FType>
void findn() {
    cout << typeid(FType).name() << " => ";
    FType pi2 = FType(2) * atan(FType(1));
    FType eps2 = numeric_limits<FType>::epsilon()/2;
    FType lhs = pi2;    // pi2^1
    FType rhs = eps2;   // eps2 * 1!
    int n = 0;
    while (lhs >= rhs) {
        lhs *= pi2;
        rhs *= ++n + 1;
    }
    cout << "n: " << n << ", lhs: " << lhs << ", rhs: " << rhs << endl;
}

int main(int argc, char* argv[]) {
    findn<float>();
    findn<double>();
}

/* Output:
float => n: 12, lhs: 354.453, rhs: 371.159
double => n: 21, lhs: 20636.6, rhs: 124789
*/
