// hdsin.cpp: Calculates sin(x) as a double. x is
// reduced to the interval [-pi/2, pi/2]. Uses
// the first 11 terms of the Taylor's Series (more
// would add nothing). Precomputes the coefficients.
// Uses Horner's rule.

#include <iostream>
#include <limits>
#include "../ieee.h"
#include "../ulps.h"
using namespace std;

double pi = 3.14159265358979323846;
const int k = 11;

// The coefficients of the Taylor's series
double c[k];

void coeff() {
   double den = 1.0;
   int fact = 1;  // The denominator contains fact!
   c[0] = den;

   for (int i = 1; i < k; ++i) {
      den *= ++fact;
      den *= ++fact;
      c[i] = 1.0/den;
      if (i%2)
         c[i] = -c[i];
      cout << "c[" << i << "] = " << c[i] << endl;
   }
}

double mysine(double x) {
   int n = int(x/pi + 0.5*sign(x));
   x = x - n*pi;
   double y = x*x;

   // Compute c[0] + c[1]*y + ... + c[6]*y^6
   double sum = c[k-1];
   for (int i = k-2; i >= 0; --i)
      sum = sum*y + c[i];

   sum = x*sum;
   return (n%2) ? -sum : sum;
}

int main() {
   coeff();
   cout.precision(numeric_limits<double>::digits10);
   cout << mysine(0.0) << " (" << sin(0.0) << ")\n";
   cout << mysine(pi/2.0) << " (" << sin(pi/2.0) << ")\n";
   cout << mysine(pi) << " (" << sin(pi) << ")\n";
   cout << mysine(3.0f*pi/2.0) << " (" << sin(3.0*pi/2.0) << ")\n";
   cout << mysine(22.0) << " (" << sin(22.0) << ")\n";
   cout << mysine(10.0e8) << " (" << sin(10.0e8) << ")\n";
   cout << ulps(mysine(22.0), sin(22.0)) << endl;
}
