// Uses Horner's rule to evaluate a polynomial
// Usage:
//    horners x a[n] a[n-1] ... a[0]
// Uses two versions

#include <algorithm>
#include <cstdlib>
#include <cstddef>
#include <iostream>
#include <vector>
using namespace std;

// A recursive Horner's Rule
template<class Iter>
double poly(Iter begin, Iter end, double x) {
   if (begin == end)
      return 0.0;
   else {
      Iter c = --end;
      return *c + x*poly(begin,end,x);
   }
}

double poly2(double c[], size_t n, double x) {
   double sum = c[0];   // This is really the nth coefficient
   for (size_t i = 1; i < n; ++i)
      sum = c[i] + x*sum;
   return sum;
}

int main(int argc, char* argv[]) {
   if (argc >= 4) {
      double x = atof(argv[1]);
      double* p_coeff = new double[argc-2];
      for (int i = 2; i < argc; ++i)
         p_coeff[i-2] = atof(argv[i]);
      cout << poly(p_coeff, p_coeff+argc-2, x) << endl;
      cout << poly2(p_coeff, argc-2, x) << endl;

      vector<double> v(p_coeff, p_coeff+argc-2);
      cout << poly(v.begin(), v.end(), x) << endl;
   }
}


