// newton.cpp: Root finding by Newton's Method
#include <cmath>
#include <cstdlib>
#include <iostream>
#include <limits>
#include "ieee2.h"
#include "ulps2.h"
using namespace std;

double f(double x) {
   return x*x - 5.0*x + 4.0;
}

double df(double x) {
   return 2.0*x - 5.0;
}

double g(double x) {
   return x - f(x)/df(x);  // Newton's formula
}

int main(int argc, char* argv[]) {
   if (argc > 1) {
      double guess = atof(argv[1]);
      double eps = numeric_limits<double>::epsilon();
      double small = numeric_limits<double>::denorm_min();
      cout.precision(numeric_limits<double>::digits10+2);

      for (int i = 0; i < 50; ++i) {
         double next = g(guess);
         cout << i << ": " << next << " (" << ulps(next,guess) << ")\n";

         // Check for near zero just in case
         double fval = abs(f(next));
         if (fval <= small) {
            cout << "near zero (" << fval << ")\n";
            break;
         }

         // Check for FP numbers close enough (1-2 ulps)
         if (abs(next - guess) <= abs(guess)*eps)
            break;
         guess = next;
      }
   }
}

