// quad2a.cpp (uses double precision)
// Solves the equation x^2 + bx + c = 0 (i.e., a == 1)
// This version avoids potential catastrophic cancellation
// and computes r2 as r1/c.

#include <cfloat>
#include <cmath>
#include <iostream>
using namespace std;

int main() {
   cout.precision(DBL_DIG-1);
   cout.setf(ios::showpoint);
   cout.setf(ios::scientific);
   double b = -1.0e5f;
   double c = 1.0f;

   double d = b*b - 4.0f*c;
   if (d >= 0) {
      // Real roots
      double e = sqrt(d), r1;
      if (b >= 0)
         r1 = (-b - e) / 2.0f;
      else
         r1 = (-b + e) / 2.0f;
      double r2 = c/r1;
      cout << "r1 = " << r1 << ", r2 = " << r2 << endl;
   }
   else {
      // Complex roots
      double p = -b / 2.0f;
      double q = sqrt(-d) / 2.0f;
      cout << "r1 = " << p << " + " << q << "i\n";
      cout << "r2 = " << p << " - " << q << "i\n";
   }
}

/* Output:
r1 = 9.99999999900000e+04, r2 = 1.00000000010000e-05
*/
