// secant.cpp: Root finding by the Secant Method
#include <cmath>
#include <cstdlib>
#include <iostream>
#include <limits>
#include "ieee2.h"
#include "ulps2.h"
using namespace std;

double f(double x) {
   return x*x - 5.0*x + 4.0;
}

int main(int argc, char* argv[]) {
   if (argc > 2) {
      double x1 = atof(argv[1]);
      double fx1 = f(x1);
      double x2 = atof(argv[2]);
      double fx2 = f(x2);
      double eps = numeric_limits<double>::epsilon();
      cout.precision(numeric_limits<double>::digits10+2);

      for (int i = 0; i < 50; ++i) {
//         if (abs(fx2) > abs(fx1)) {
//            swap(x1, x2);
//            swap(fx1, fx2);
//         }
         double next = x2 - fx2*(x2 - x1)/(fx2 - fx1);
         double fnext = f(next);
         cout << i << ": " << next << " (" << fnext << ") (ulps = " << ulps(x1, x2) << ")\n";

         // Check for zero just in case
         if (abs(fnext) == 0.0 || abs(next - x2) <= abs(next)*eps)
            break;

         x1 = x2;
         fx1 = fx2;
         x2 = next;
         fx2 = fnext;
      }
   }
}

